/*
 * Copyright (C) 2020 - 2022 SG Wireless Limited - All Rights Reserved
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 *
 */
/**
 * @file ch376.c
 * @brief
 */
/***********************************************Include***********************************************/
#include <stdio.h>

#include "CH376INC.h"
#include "microboard.h"
#include "ch376.h"
#include "ch376_fs.h"
#include "hc595.h"
#include "nrf_delay.h"
#include "nrf_drv_spi.h"
#include "nrf_gpiote.h"
#include "nrf_drv_gpiote.h"

/* FreeRTOS */
#include "FreeRTOS.h"
#include "task.h"

/* Log */
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

#define USE_SPIM 1
#define USE_INT  1

#define CH376_CS_PIN    USB_SS
#define CH376_SCK_PIN   USB_SCK
#define CH376_MISO_PIN  USB_MISO
#define CH376_MOSI_PIN  USB_MOSI

#define CH376_INT_PIN   USB_INT

/***********************************Global Definitions and Declarations*******************************/
#ifdef USE_SPIM
nrfx_spim_t hspi1 = NRFX_SPIM_INSTANCE(3);
#else
nrf_drv_spi_t hspi1 = NRF_DRV_SPI_INSTANCE(1);
#endif
/***********************************Constant / Macro Definitions**************************************/
/*******************************************Type Declarations*****************************************/
/******************************************Function Declarations**************************************/
/******************************************Variable Declarations**************************************/
/*******************************************Function Prototype****************************************/

/* Check CH376 IRQ (INT# low level)*/
bool Query376Interrupt(void) {
    bool res = nrf_gpio_pin_read(CH376_MISO_PIN) ? false : true;
    return res;
}

//static void input_iqr_handle(nrfx_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
//{
//    if(nrf_gpio_pin_read(CH376_INT_PIN) == 0)
//    {
//        irq_flag = true;
//    }
//    else {
//        irq_flag = false;
//    }
//}

void ch376_usb_power(bool state) {
    if(state == true) {
        hc595_write_bit(HC595_OUT_CH6, true);
    }
    else {
        hc595_write_bit(HC595_OUT_CH6, false);
    }
}

ch376_err_t ch376_write_data(uint8_t data) {
    uint8_t err     = 0;
    uint8_t wbuf[2] = {0}, rbuf[2] = {0};

    wbuf[0] = data;
#ifdef USE_SPIM
    nrfx_spim_xfer_desc_t const spim_xfer_desc =
    {
        .p_tx_buffer = wbuf,
        .tx_length   = 1,
        .p_rx_buffer = rbuf,
        .rx_length   = 1,
    };
    err = nrfx_spim_xfer(&hspi1, &spim_xfer_desc, 0);
#else
    err = nrf_drv_spi_transfer(&hspi1, wbuf, 1, rbuf, 1);
    APP_ERROR_CHECK(err);
#endif
    if (err != NRF_SUCCESS) {
        err = CH376_SPI_ERROR;
    } else {
        err = CH376_SUCCEED;
    }
    return err;
}

ch376_err_t ch376_read_data(uint8_t *data) {
    uint8_t err     = 0;
    uint8_t wbuf[2] = {0}, rbuf[2] = {0};

    wbuf[0] = 0xff;
#ifdef USE_SPIM
    nrfx_spim_xfer_desc_t const spim_xfer_desc =
    {
        .p_tx_buffer = wbuf,
        .tx_length   = 1,
        .p_rx_buffer = rbuf,
        .rx_length   = 1,
    };
    err = nrfx_spim_xfer(&hspi1, &spim_xfer_desc, 0);
#else
    err = nrf_drv_spi_transfer(&hspi1, wbuf, 1, rbuf, 1);
    APP_ERROR_CHECK(err);
#endif
    if (err != NRF_SUCCESS) {
        err = CH376_SPI_ERROR;
    } else {
        err = CH376_SUCCEED;
    }

    *data = rbuf[0];
    return err;
}

ch376_err_t ch376_write_256(uint8_t* data, uint8_t len) {
    uint8_t err     = 0;
    uint8_t wbuf[256] = {0}, rbuf[256] = {0};

    memcpy(wbuf, data, len);
#ifdef USE_SPIM
    nrfx_spim_xfer_desc_t const spim_xfer_desc =
    {
        .p_tx_buffer = wbuf,
        .tx_length   = len,
        .p_rx_buffer = rbuf,
        .rx_length   = len,
    };
    err = nrfx_spim_xfer(&hspi1, &spim_xfer_desc, 0);
#else
    err = nrf_drv_spi_transfer(&hspi1, wbuf, len, rbuf, len);
    APP_ERROR_CHECK(err);
#endif
    if (err != NRF_SUCCESS) {
        err = CH376_SPI_ERROR;
    } else {
        err = CH376_SUCCEED;
    }
    return err;
}

ch376_err_t ch376_read_256(uint8_t *data, uint8_t len) {
    uint8_t err     = 0;
    uint8_t wbuf[256] = {0}, rbuf[256] = {0};

    memset(wbuf, 0xff, len);
#ifdef USE_SPIM
    nrfx_spim_xfer_desc_t const spim_xfer_desc =
    {
        .p_tx_buffer = wbuf,
        .tx_length   = len,
        .p_rx_buffer = rbuf,
        .rx_length   = len,
    };
    err = nrfx_spim_xfer(&hspi1, &spim_xfer_desc, 0);
#else
    err = nrf_drv_spi_transfer(&hspi1, wbuf, len, rbuf, len);
    APP_ERROR_CHECK(err);
#endif
    if (err != NRF_SUCCESS) {
        err = CH376_SPI_ERROR;
    } else {
        err = CH376_SUCCEED;
    }

    *data = rbuf[0];
    return err;
}

void xEndCH376Cmd(void) {
    nrf_gpio_pin_set(CH376_CS_PIN);
    //nrf_delay_ms(1);
}

void xWriteCH376Cmd(uint8_t cmd) {
    nrf_gpio_pin_clear(CH376_CS_PIN);

    ch376_write_data(cmd);
    //nrf_delay_us(10);
}

uint8_t xReadCH376Data(void) {
    uint8_t data;

    //nrf_delay_us(10);
    ch376_read_data(&data);

    return data;
}

void xWriteCH376Data(UINT8 data) {
    ch376_write_data(data);
    //nrf_delay_us(10);
}

ch376_err_t ch376_transfer_check(void) {
    xWriteCH376Cmd(CMD11_CHECK_EXIST);
    uint8_t res = 0;
    xWriteCH376Data(0x55);
    res = xReadCH376Data();
    xEndCH376Cmd();
    if (res != 0xAA) {
        return CH376_HARDWARE_ERROR;
    }
    return CH376_SUCCEED;
}

static void ch376_spi_init(void) {
#ifdef USE_SPIM
    nrfx_spim_config_t  spi_config =NRFX_SPIM_DEFAULT_CONFIG;
    spi_config.mosi_pin  = CH376_MOSI_PIN;
    spi_config.miso_pin  = CH376_MISO_PIN;
    spi_config.sck_pin   = CH376_SCK_PIN;
    spi_config.frequency = NRF_SPIM_FREQ_16M;
    APP_ERROR_CHECK(nrfx_spim_init(&hspi1, &spi_config, NULL, NULL));
#else
    nrf_drv_spi_config_t spi_config = NRF_DRV_SPI_DEFAULT_CONFIG;
    // spi_config.ss_pin   = CH376_CS_PIN;
    spi_config.mosi_pin  = CH376_MOSI_PIN;
    spi_config.miso_pin  = CH376_MISO_PIN;
    spi_config.sck_pin   = CH376_SCK_PIN;
    spi_config.frequency = NRF_DRV_SPI_FREQ_8M;
    APP_ERROR_CHECK(nrf_drv_spi_init(&hspi1, &spi_config, NULL, NULL));
#endif
    nrf_gpio_cfg_output(CH376_CS_PIN);
    nrf_gpio_pin_set(CH376_CS_PIN);
}

typedef struct _FILE_NAME {
    UINT32 DirStartClust; /* The starting cluster number of the directory where the file is located */
    UINT32 Size;          /* file length */
    UINT8 Name[8 + 1 + 3 + 1]; /* File name, a total of 8+3 bytes, delimiter, end character, because it does not contain the upper-level directory name, so it is a relative path */
    UINT8 Attr;                /* file attributes */
} FILE_NAME;
#define MAX_FILE_COUNT 40
FILE_NAME FileNameBuffer[MAX_FILE_COUNT]; /* file name structure */
UINT16 FileCount;
UINT8 buf[2048];
/* Example: List all files in the directory with the specified serial number */
/* The input parameter index refers to the serial number of the directory in the structure*/
UINT8 ListFile(UINT8 index)
{
    UINT8 s;
    P_FAT_DIR_INFO pDir;
    PUINT8 pNameBuf;
    UINT32 CurrentDirStartClust; /* Save the starting cluster number of the current directory to speed up file enumeration and opening */

    CH376WriteVar32(VAR_START_CLUSTER, FileNameBuffer[index].DirStartClust); /* Set the initial cluster number of the upper-level directory of the current directory to the current cluster number, which is equivalent to opening the upper-level directory */
    NRF_LOG_INFO("List Directory: %s", FileNameBuffer[index].Name);
    s = CH376FileOpen(FileNameBuffer[index].Name); /* Open the directory, just to get the starting cluster number of the directory to improve speed */
    NRF_LOG_INFO("open ret:0x%x", s);
    if (s == USB_INT_SUCCESS)
        return (ERR_FOUND_NAME); /* The directory should be opened, but the return result is that the file is opened */
    else if (s != ERR_OPEN_DIR)
        return (s);
    if (index) {
        CurrentDirStartClust = CH376ReadVar32(VAR_START_CLUSTER); /* Not the root directory, get the starting cluster number of the directory */
    }
    else {
        CurrentDirStartClust = 0;                             /* Is the root directory */
        CH376DirCreate("testd");
    }
    CH376FileClose(FALSE);                                    /* For the root directory must be closed */

    CH376WriteVar32(VAR_START_CLUSTER, CurrentDirStartClust); /* The starting cluster number of the current directory, which is equivalent to opening the current directory */
    /* Set the file name of the file to be operated, wildcards support all files and subdirectories */
    CH376SetFileName("*");
    xWriteCH376Cmd(CMD0H_FILE_OPEN); /* Enumerate files and directories */
    xEndCH376Cmd();

    while (1) {
        s = Wait376Interrupt();
        if (s == USB_INT_DISK_READ) { /* request data read */
            /* During the file enumeration process, other operating commands that may cause interruption cannot be executed. For example, if you need to obtain a long file name, you can save the enumerated file name and obtain its long file name after the enumeration ends */
            CH376ReadBlock(buf); /* Read the FAT_DIR_INFO structure of the enumerated file, the return length is always sizeof( FAT_DIR_INFO ) */
            xWriteCH376Cmd(CMD0H_FILE_ENUM_GO); /* Continue to enumerate files and directories, issue the next command first and then analyze the data read from the uplink, so that the CH376 and the single-chip microcomputer can work at the same time, increasing the speed */
            xEndCH376Cmd();
            pDir = (P_FAT_DIR_INFO)buf;     /* Current file directory information */

            if (pDir->DIR_Name[0] != '.') { /* If it is not the name of the current level or the upper level directory, continue, otherwise it must be discarded and not processed*/
                if (pDir->DIR_Name[0] == 0x05)
                    pDir->DIR_Name[0] = 0xE5;                            /* special character replacement */

                //if (pDir->DIR_Name[8] == 'H' && pDir->DIR_Name[9] == ' ' /* Example of comparing file extensions to analyze file types */
                //    || pDir->DIR_Name[8] == 'T' && pDir->DIR_Name[9] == 'X' && pDir->DIR_Name[10] == 'T') {
                //    NRF_LOG_INFO("This is a .H or .TXT file");
                //}

                if (FileCount < MAX_FILE_COUNT) {               /* The filename structure buffer is sufficient */
                    pNameBuf = FileNameBuffer[FileCount].Name; /* Filename buffer in the filename structure */
                    for (s = 0; s < 11; s++) {                  /* Copy the file name, the length is 11 characters */
                        if (pDir->DIR_Name[s] != 0x20) {        /* Valid characters */
                            if (s == 8) {                       /* Handle extensions */
                                *pNameBuf = '.';                /* Delimiter */
                                pNameBuf++;
                            }
                            *pNameBuf = pDir->DIR_Name[s]; /* Copy one character of the filename */
                            pNameBuf++;
                        }
                    }
                    *pNameBuf = 0; /* The terminator of the full path to the current filename */

                    FileNameBuffer[FileCount].DirStartClust = CurrentDirStartClust; /* Record the starting cluster number of the current directory to speed up file opening */
                    FileNameBuffer[FileCount].Attr = pDir->DIR_Attr; /* Record file properties */
                    FileNameBuffer[FileCount].Size = pDir->DIR_FileSize;

                    if (pDir->DIR_Attr & ATTR_DIRECTORY) {
                        NRF_LOG_INFO("Dir %4d#: %s\n", FileCount, FileNameBuffer[FileCount].Name); /* Judgment is a directory name */
                    }
                    else {
                        NRF_LOG_INFO("File %4d#: %s,size:%d\n", FileCount, FileNameBuffer[FileCount].Name, pDir->DIR_FileSize); /* Judgment is the file name */
                    }
                    FileCount++;                 /* subdirectory count */
                } else { /* Filename structure buffer too small, insufficient number of structures */
                    NRF_LOG_INFO("FileName Structure Full\n");
                    s = Wait376Interrupt();

                    CH376EndDirInfo(); /* After getting the FAT_DIR_INFO structure */
                    break;             /* Forcibly terminate enumeration */
                }
            }
        } else {
            if (s == ERR_MISS_FILE) {
                s = USB_INT_SUCCESS; /* No more matching files found */
            }
            NRF_LOG_INFO("no file\r\n");
            break;
        }
    }
    return (s);
}

UINT8 ListAll(void) /* Enumerate all files and directories in the entire U disk with a breadth-first algorithm */
{
    UINT8 s;
    UINT16 OldFileCount;
    UINT16 RealReadCount;

    FileNameBuffer[0].Name[0] = '/'; /* The root directory is a complete path name, and all paths are relative except that the root directory is an absolute path */
    FileNameBuffer[0].Name[1]       = 0;
    FileNameBuffer[0].DirStartClust = 0;              /* The starting cluster number of the root directory */
    FileNameBuffer[0].Attr          = ATTR_DIRECTORY; /* The root directory is also a directory, saved as the first record */

    for (OldFileCount = 0, FileCount = 1; OldFileCount < FileCount; OldFileCount++) { /* There are still newly enumerated file name structures that have not been analyzed, and FileCount is changing */
        if (FileNameBuffer[OldFileCount].Attr & ATTR_DIRECTORY) { /* If it is a directory, continue to search in depth */
            s = ListFile(OldFileCount); /* Enumerate directories, save records into structure, FileCount may change */
            if (s != USB_INT_SUCCESS) {
                return (s);
            }
        }
    }
}

void ch376_init(void)
{
    uint32_t err_code;
    uint8_t data = 0;

    ch376_spi_init();
    ch376_usb_power(true);

    xWriteCH376Cmd(CMD_RESET_ALL);
    xEndCH376Cmd();
    nrf_delay_ms(200);

    xWriteCH376Cmd(CMD_GET_IC_VER);
    data = xReadCH376Data();
    xEndCH376Cmd();
    NRF_LOG_INFO("Version:%d", data & 0x3F);

    uint8_t res = ch376_transfer_check();
    if(res != CH376_SUCCEED) {
        NRF_LOG_INFO("transfer check error");
    }

    // Set SDO as INT pin
    xWriteCH376Cmd(CMD_SET_SDO_INT);
    xWriteCH376Data(0x16);
    xWriteCH376Data(0x90);
    xEndCH376Cmd();

    xWriteCH376Cmd(CMD_SET_USB_MODE);
    xWriteCH376Data(0x05);
    nrf_delay_us(20);
    res = xReadCH376Data();
    xEndCH376Cmd();
    if(res != CMD_RET_SUCCESS) {
        NRF_LOG_INFO("mode set status:%x", res);
    }
}

static void usb_disk_file_test(void)
{
    UINT8 s;
    UINT16  RealReadCount = 0, RealWriteCount = 0;
    UINT8 writedata[12] = "hello ch376";

    NRF_LOG_INFO("Create .txt file");
    s = CH376FileCreate("/HELLO.TXT");

    NRF_LOG_INFO("Open, write .txt file");
    if ( s == USB_INT_SUCCESS ) {
        CH376ByteWrite( writedata, sizeof(writedata), &RealWriteCount );
        NRF_LOG_INFO("s:0x%x write file %d byte", s, RealWriteCount);
        CH376FileClose( TRUE );
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }

    NRF_LOG_INFO("Open, read .txt file");
    s = CH376FileOpen("/HELLO.TXT");
    if ( s == USB_INT_SUCCESS ) {
        s = CH376ByteRead( buf, sizeof(buf), &RealReadCount );
        NRF_LOG_INFO("s:0x%x read file %d byte", s, RealReadCount);
        CH376FileClose( TRUE );
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }
}


#define BUF_SIZE  1024*10
UINT8  writedata[BUF_SIZE], readbuf[BUF_SIZE];

static void usb_disk_transfer_speed_test(void)
{
#define BYTE_TEST 1
#define SEC_TEST  0
    UINT8   s;
    UINT32  filesize;
    UINT16  readcnt = 0;
    
    TickType_t xTickCount,xLastTickCount;
    int32_t  times = 0, speed = 0;

    memset(writedata, 'A', BUF_SIZE);

#if BYTE_TEST //Byte test
    #define BYTEW_CNT 1
    UINT16  RealReadCount = 0, RealWriteCount = 0;

    xLastTickCount = xTaskGetTickCount();
    NRF_LOG_INFO("now systick:%d", xLastTickCount);

    //Write
    s = CH376FileOpen("/BYTE.TXT");
    if(s = ERR_MISS_FILE) {
        s = CH376FileCreate("/BYTE.TXT");
    }
    if ( s == USB_INT_SUCCESS ) {
        NRF_LOG_INFO("Open, write .txt file");
        for(UINT8 i = 0; i < BYTEW_CNT; i++) {
            CH376ByteWrite(writedata, sizeof(writedata), &RealWriteCount);
        }
        NRF_LOG_INFO("write %d byte, total: %d", RealWriteCount, RealWriteCount*BYTEW_CNT);

        CH376FileClose( TRUE );
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }

    xTickCount = xTaskGetTickCount();
    times = xTickCount - xLastTickCount;
    speed = RealWriteCount * BYTEW_CNT * 1000 / 1024 / times;
    NRF_LOG_INFO("write tick count: %d, speed:%d k byte/s", times, speed);

    filesize = CH376GetFileSize( );
    NRF_LOG_INFO( "File size = %ld\n", filesize );

    xLastTickCount = xTaskGetTickCount();
    vTaskDelay(20);
    xTickCount = xTaskGetTickCount();
    NRF_LOG_INFO("delay cnt:%d\n", xTickCount - xLastTickCount);

    //Read
    s = CH376FileOpen("/BYTE.TXT");
    if ( s == USB_INT_SUCCESS ) {
        NRF_LOG_INFO("Open, read .txt file");
        
        xLastTickCount = xTaskGetTickCount();
        NRF_LOG_INFO("now systick:%d", xLastTickCount);

        do {
            CH376ByteRead( readbuf, sizeof(readbuf), &RealReadCount );
            readcnt++;
        }while(RealReadCount == sizeof(readbuf));
        NRF_LOG_INFO("read %d byte, last read %d byte", sizeof(readbuf), RealReadCount);
        NRF_LOG_INFO("read cnt: %d, total byte: %d", readcnt-1, readcnt*sizeof(readbuf)+RealReadCount);
        
        CH376FileClose( TRUE );
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }
    xTickCount = xTaskGetTickCount();
    times = xTickCount - xLastTickCount;
    speed = ((readcnt - 1) * sizeof(readbuf) + RealReadCount) * 1000 / 1024 / times;
    NRF_LOG_INFO("read tick count: %d, speed:%d k Byte/s", times, speed);
#endif

#if SEC_TEST //Sec test
    #define SECW_CNT 1
    UINT16 sec_num;
    UINT8  RealReadCount = 0, RealWriteCount = 0;

    xLastTickCount = xTaskGetTickCount();
    NRF_LOG_INFO("now systick:%d", xLastTickCount);

    //Write
    s = CH376FileOpen("/SEC.TXT");
    if(s = ERR_MISS_FILE) {
        s = CH376FileCreate("/SEC.TXT");
    }

    if ( s == USB_INT_SUCCESS ) {
        NRF_LOG_INFO("Open, write .txt file");
        sec_num = BUF_SIZE / 512;
        for(UINT8 i = 0; i < SECW_CNT; i++) {
            CH376SecWrite(writedata, sec_num, &RealWriteCount);
        }
        CH376FileClose( TRUE );
        NRF_LOG_INFO("write cnt: %d, total:%d", RealWriteCount, RealWriteCount*512*SECW_CNT);
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }
    xTickCount = xTaskGetTickCount();
    times = xTickCount - xLastTickCount;
    speed = RealWriteCount * 512 * SECW_CNT * 1000 / 1024 / times;
    NRF_LOG_INFO("write tick count: %d, speed:%d kb/s", times, speed);

    filesize = CH376GetFileSize( );
    NRF_LOG_INFO( "File size = %ld\n", filesize );
    xLastTickCount = xTaskGetTickCount();
    vTaskDelay(20);
    xTickCount = xTaskGetTickCount();
    NRF_LOG_INFO("delay cnt:%d\n", xTickCount - xLastTickCount);
    xLastTickCount = xTaskGetTickCount();
    NRF_LOG_INFO("now systick:%d", xLastTickCount);

    //Read
    s = CH376FileOpen("/SEC.TXT");
    if ( s == USB_INT_SUCCESS ) {
        NRF_LOG_INFO("Open, read .txt file");
        CH376SecLocate( 0 );
        for(UINT8 i = 0; i < SECW_CNT; i++) {
            CH376SecRead( readbuf, sec_num, &RealReadCount );
        }
        CH376FileClose( TRUE );
        NRF_LOG_INFO("read cnt: %d, total byte: %d", RealReadCount, 512 * sec_num * SECW_CNT);
    }
    else {
        NRF_LOG_INFO("error:0x%x", s);
    }
    xTickCount = xTaskGetTickCount();
    times = xTickCount - xLastTickCount;
    speed = SECW_CNT * sec_num * 512 * 1000 / 1024 / times;
    NRF_LOG_INFO("read tick count: %d, speed:%d kb/s", times, speed);
#endif
}

void ch376_task(void *pvParams) {
    UINT8 i, s;

    NRF_LOG_INFO("USB Task start");

    while (1) {
        NRF_LOG_INFO("Wait Connected");
        while (CH376DiskConnect() != USB_INT_SUCCESS) { /* Check whether the U disk is connected and wait for the U disk to be inserted. For the SD card, the single chip microcomputer can directly query the plug status pin of the SD card socket */
            vTaskDelay(pdMS_TO_TICKS(100));
        }

        xWriteCH376Cmd(CMD_GET_DEV_RATE);
        xWriteCH376Data(0x07);
        nrf_delay_us(20);
        s = xReadCH376Data();
        xEndCH376Cmd();
        NRF_LOG_INFO("speed mode:0x%x", s);

        xWriteCH376Cmd(CMD_SET_USB_MODE);
        xWriteCH376Data(0x06);
        nrf_delay_us(20);
        s = xReadCH376Data();
        xEndCH376Cmd();
        if(s != CMD_RET_SUCCESS) {
            NRF_LOG_INFO("mode set status:0x%x", s);
        }
        
        for (i = 0; i < 100; i++) { /* The longest waiting time, 100*50mS */
            vTaskDelay(250);
            s = CH376DiskMount(); /* Initialize the disk and test whether the disk is ready */
            NRF_LOG_INFO("Ready? ret:0x%x", s);
            if (s == USB_INT_SUCCESS) break; /* ready */
            else if (s == ERR_DISK_DISCON) break; /* Disconnect detected, re-detected and timed */
            if (i >= 5)
                break; /* Some U disks always return not ready, but it can be ignored, as long as it establishes a connection MOUNTED and tries 5*50mS */
        }
        if (s == ERR_DISK_DISCON) { /* Disconnect detected, re-detected and timed */
            NRF_LOG_INFO("Device gone\n");
            continue;
        }
        if (CH376GetDiskStatus() < DEF_DISK_MOUNTED) { /* Unknown USB devices, such as USB keyboards, printers, etc. */
            NRF_LOG_INFO("Unknown device\n");
            continue;
        }

        i = CH376ReadBlock(buf); /* If necessary, the data block CH376_CMD_DATA.DiskMountInq can be read, and the length is returned */
        if (i == sizeof(INQUIRY_DATA)) { /* U disk manufacturer and product information */
            buf[i]         = 0;
            INQUIRY_DATA t = *(P_INQUIRY_DATA)buf;
            NRF_LOG_INFO("UdiskInfo: %s\n", ((P_INQUIRY_DATA)buf)->VendorIdStr);
        }

        //usb_disk_file_test();
        usb_disk_transfer_speed_test();

        NRF_LOG_INFO("List all file");
        s = ListAll(); /* Enumerate all files and directories in the entire U disk */
        NRF_LOG_FLUSH();
        while ( CH376DiskConnect( ) == USB_INT_SUCCESS ) {  /* Check whether the U disk is connected, wait for the U disk to be pulled out */
            vTaskDelay(pdMS_TO_TICKS(100));
        }
        NRF_LOG_INFO("Disconnected");
        vTaskDelay(pdMS_TO_TICKS(100));
    }
}