/*
* Copyright (C) 2020 - 2022 SG Wireless Limited - All Rights Reserved
*
* Unauthorized copying of this file, via any medium is strictly prohibited
* Proprietary and confidential
*
*/
/**
* @file ch376_fs.h
* @brief 
*/
#ifndef CH376_FS_H
#define CH376_FS_H
#ifdef __cplusplus
extern "C" {
#endif
/***********************************************Include***********************************************/
#include "ch376.h"
/***********************************Global Definitions and Declarations*******************************/
/***********************************Constant / Macro Definitions**************************************/
/*******************************************Type Declarations*****************************************/
/******************************************Function Declarations**************************************/
/******************************************Variable Declarations**************************************/
#define	STRUCT_OFFSET( s, m )	( (UINT8)( & ((s *)0) -> m ) )	/* Define a macro to obtain the relative offset address of a structure member */

#ifdef	EN_LONG_NAME
#ifndef	LONG_NAME_BUF_LEN
#define	LONG_NAME_BUF_LEN	( LONG_NAME_PER_DIR * 20 )	/* Self-defined long file name buffer length, the minimum value is LONG_NAME_PER_DIR*1 */
#endif
#endif

#ifndef	EN_SECTOR_ACCESS
#define	EN_SECTOR_ACCESS	1
#endif


UINT8	CH376ReadBlock( PUINT8 buf );  /* Read a block of data from the receive buffer of the current host endpoint and return the length */

UINT8	CH376WriteReqBlock( PUINT8 buf );  /* Write the requested data block to the internal designated buffer and return the length */

void	CH376WriteHostBlock( PUINT8 buf, UINT8 len );  /* Write a block of data to the send buffer of the USB host endpoint */

void	CH376WriteOfsBlock( PUINT8 buf, UINT8 ofs, UINT8 len );  /* Write a data block to the specified offset address of the internal buffer */

void	CH376SetFileName( PUINT8 name );  /* Set the filename of the file to be operated on */

UINT32	CH376Read32bitDat( void );  /* Read 32-bit data from CH376 chip and end the command */

UINT8	CH376ReadVar8( UINT8 var );  /* Read the 8-bit variable inside the CH376 chip */

void	CH376WriteVar8( UINT8 var, UINT8 dat );  /* Write the 8-bit variable inside the CH376 chip */

UINT32	CH376ReadVar32( UINT8 var );  /* Read the 32-bit variables inside the CH376 chip */

void	CH376WriteVar32( UINT8 var, UINT32 dat );  /* Write the 32-bit variables inside the CH376 chip */

void	CH376EndDirInfo( void );  /* After calling CH376DirInfoRead to get the FAT_DIR_INFO structure, CH376 should be notified to end */

UINT32	CH376GetFileSize( void );  /* Read current file length */

UINT8	CH376GetDiskStatus( void );  /* Get the working status of disks and filesystems */

UINT8	CH376GetIntStatus( void );  /* Get interrupt status and cancel interrupt request */

#ifndef	NO_DEFAULT_CH376_INT
UINT8	Wait376Interrupt( void );  /* Wait for CH376 interrupt (INT# low level), return interrupt status code, return ERR_USB_UNKNOWN when timeout */
#endif

UINT8	CH376SendCmdWaitInt( UINT8 mCmd );  /* After sending the command code, wait for the interrupt */

UINT8	CH376SendCmdDatWaitInt( UINT8 mCmd, UINT8 mDat );  /* After sending the command code and one byte of data, wait for the interrupt */

UINT8	CH376DiskReqSense( void );  /* Check USB memory for errors */

UINT8	CH376DiskConnect( void );  /* 不支持SD卡,Check whether the U disk is connected, SD card is not supported */

UINT8	CH376DiskMount( void );  /* Initialize the disk and test whether the disk is ready */

UINT8	CH376FileOpen( PUINT8 name );  /* Open a file or directory (folder) in the root directory or the current directory */

UINT8	CH376FileCreate( PUINT8 name );  /* Create a new file in the root directory or the current directory, if the file already exists, delete it first */

UINT8	CH376DirCreate( PUINT8 name );  /* Create a new directory (folder) under the root directory and open it, if the directory already exists, open it directly */

UINT8	CH376SeparatePath( PUINT8 path );  /* Separate the last-level file name or directory (folder) name from the path, and return the byte offset of the last-level file name or directory name */

UINT8	CH376FileOpenDir( PUINT8 PathName, UINT8 StopName );  /* Open a file in a multi-level directory or the parent directory of a directory, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
/* StopName points to the last level file name or directory name */

UINT8	CH376FileOpenPath( PUINT8 PathName );  /* Open files or directories (folders) under multi-level directories, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */

UINT8	CH376FileCreatePath( PUINT8 PathName );  /* Create a file under a multi-level directory, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */

#ifdef	EN_DIR_CREATE
UINT8	CH376DirCreatePath( PUINT8 PathName );  /* Create a new directory (folder) under the multi-level directory and open it, support multi-level directory paths, support path separators, and the path length does not exceed 255 characters */
#endif

UINT8	CH376FileErase( PUINT8 PathName );  /* Delete the file, if it has been opened, delete it directly, otherwise the file will be opened first and then deleted, support multi-level directory path */

UINT8	CH376FileClose( UINT8 UpdateSz );  /* Close the currently opened file or directory (folder) */

UINT8	CH376DirInfoRead( void );  /* Read the directory information of the current file */

UINT8	CH376DirInfoSave( void );  /* Save the directory information of the file */

UINT8	CH376ByteLocate( UINT32 offset );  /* Move the current file pointer in bytes */

UINT8	CH376ByteRead( PUINT8 buf, UINT16 ReqCount, PUINT16 RealCount );  /* Read a block of data from the current position in bytes */

UINT8	CH376ByteWrite( PUINT8 buf, UINT16 ReqCount, PUINT16 RealCount );  /* Write a data block to the current location in bytes */

#ifdef	EN_DISK_QUERY

UINT8	CH376DiskCapacity( PUINT32 DiskCap );  /* Query disk physical capacity, number of sectors */

UINT8	CH376DiskQuery( PUINT32 DiskFre );  /* Query disk remaining space information, number of sectors */

#endif

UINT8	CH376SecLocate( UINT32 offset );  /* Move the current file pointer in units of sectors */

#ifdef	EN_SECTOR_ACCESS

UINT8	CH376DiskReadSec( PUINT8 buf, UINT32 iLbaStart, UINT8 iSectorCount );  /* Read data blocks of multiple sectors from the U disk to the buffer, does not support SD cards */

UINT8	CH376DiskWriteSec( PUINT8 buf, UINT32 iLbaStart, UINT8 iSectorCount );  /* Write the data blocks of multiple sectors in the buffer to the U disk, SD card is not supported */

UINT8	CH376SecRead( PUINT8 buf, UINT8 ReqCount, PUINT8 RealCount );  /* Read the data block from the current position in units of sectors, SD card is not supported */

UINT8	CH376SecWrite( PUINT8 buf, UINT8 ReqCount, PUINT8 RealCount );  /* Write a data block at the current position in units of sectors, SD card is not supported */

#endif

#ifdef	EN_LONG_NAME

UINT8	CH376LongNameWrite( PUINT8 buf, UINT16 ReqCount );  /* Byte-writing subroutines for long filenames */

UINT8	CH376CheckNameSum( PUINT8 DirName );  /* Computes the short filename checksum of a long filename, input in a fixed 11-byte format with no decimal separator */

UINT8	CH376LocateInUpDir( PUINT8 PathName );  /* Move the file pointer in the parent directory (folder) to the sector where the directory information of the current file is located */
/* In addition, by the way, write the LBA address of the previous sector of the sector where the current file directory information is located into the CH376 internal VAR_FAT_DIR_LBA variable (for the convenience of searching forward when collecting long file names, otherwise it needs to be moved once more) */
/* The first 12 bytes of the global buffer GlobalBuf are used */

UINT8	CH376GetLongName( PUINT8 PathName, PUINT8 LongName );  /* Obtain the corresponding long file name from the short file name or directory (folder) name */
/* You need to enter the full path PathName of the short file name, and you need to provide a buffer to receive the long file name LongName (encoded in UNICODE little endian, ending with double 0) */
/* The first 34 bytes of the global buffer GlobalBuf are used, sizeof(GlobalBuf)>=sizeof(FAT_DIR_INFO)+2 */

UINT8	CH376CreateLongName( PUINT8 PathName, PUINT8 LongName );  /* Create a new file with a long file name, return after closing the file, the LongName input path must be in RAM */
/* You need to enter the full path PathName of the short file name (please refer to the FAT specification to generate the long file name yourself), and you need to enter the long file name LongName encoded in UNICODE little endian and ending with double 0 */
/* The first 64 bytes of the global buffer GlobalBuf are used, sizeof(GlobalBuf)>=sizeof(FAT_DIR_INFO)*2 */
#endif
#ifdef __cplusplus
extern "C" }
#endif
#endif  /*ch376_fs.h*/